use bevy::{prelude::*, time::Stopwatch};

use crate::GameplaySet;

#[derive(Component)]
pub struct GameState {
    game_time: Stopwatch,
}

#[derive(Component)]
struct StateText;

pub fn state_and_ui_plugin(app: &mut App) {
    app.add_systems(
        OnEnter(crate::GameState::InGame),
        (setup_gamestate, setup_ui).in_set(GameplaySet),
    )
    .add_systems(Update, (update_time).in_set(GameplaySet));
}

fn setup_gamestate(mut commands: Commands) {
    commands.spawn(GameState {
        game_time: Stopwatch::new(),
    });
}

fn setup_ui(mut commands: Commands) {
    commands.spawn((
        TextBundle::from_sections([
            TextSection::new("Score: ", TextStyle::default()),
            TextSection::new("0", TextStyle::default()),
        ])
        .with_style(Style {
            position_type: PositionType::Absolute,
            top: Val::Px(5.0),
            right: Val::Px(5.0),
            ..default()
        }),
        StateText,
    ));
}

fn update_time(
    mut state_txt: Query<&mut Text, With<StateText>>,
    mut game_state: Query<&mut GameState>,
    time: Res<Time>,
) {
    let mut txt = state_txt.single_mut();
    let time = game_state
        .single_mut()
        .game_time
        .tick(time.delta())
        .elapsed_secs()
        * 100.;

    txt.sections[1].value = format!("{time:.0}");
}
