use crate::main_menu::main_menu_plugin;
use bevy::{
    prelude::*,
    sprite::{MaterialMesh2dBundle, Mesh2dHandle},
};
use bevy_rapier2d::prelude::*;
use drops::spawner_plugin;
use main_menu::MainMenuSet;
use player::LifeChangeEvent;

const METER: f32 = 48.;

#[derive(SystemSet, Debug, Clone, PartialEq, Eq, Hash)]
struct GameplaySet;

#[derive(States, Debug, Clone, PartialEq, Eq, Hash)]
enum GameState {
    MainMenu,
    InGame,
}

fn main() {
    let mut rapier_config = RapierConfiguration::new(METER * 2.5);
    rapier_config.timestep_mode = TimestepMode::Variable {
        max_dt: 1.0 / 60.0,
        time_scale: 1.0,
        substeps: 4,
    };
    App::new()
        .add_plugins(DefaultPlugins)
        // this does things with gravity
        .insert_resource(rapier_config)
        .insert_state(GameState::MainMenu)
        .add_plugins(RapierPhysicsPlugin::<NoUserData>::pixels_per_meter(METER))
        .add_plugins(RapierDebugRenderPlugin::default())
        .add_plugins((
            main_menu_plugin,
            spawner_plugin,
            game_state::state_and_ui_plugin,
            player::player_plugin,
        ))
        .add_systems(Startup, setup_cam)
        .add_systems(OnEnter(GameState::InGame), scene::setup_scene)
        .configure_sets(
            Update,
            (
                GameplaySet.run_if(in_state(GameState::InGame)),
                MainMenuSet.run_if(in_state(GameState::MainMenu)),
            ),
        )
        .run()
}

fn setup_cam(mut commands: Commands) {
    commands.spawn(Camera2dBundle {
        transform: Transform::from_xyz(0., 4. * METER, 0.),
        ..default()
    });
}

mod drops;
mod game_state;
mod main_menu;
mod player;
mod scene;
